"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const reader_1 = require("./reader");
const analyzer_1 = require("./analyzer");
const Mustache = require("mustache");
const fs = require("fs");
const del = require("del");
const moment = require("moment");
class Reporter {
    constructor() {
        this.theme = 'white';
        this.folderToReadReport = `${process.cwd()}/`;
        this.folderToWriteReport = `${process.cwd()}/report/gherkin-detailer/`;
        this.folderToReadTemplates = `${__dirname}/../templates/`;
        this.gherkins = [];
        this.templates = {};
        this.templatesView = {};
        this.templatePartials = {};
        this.reader = new reader_1.Reader();
        this.analyzer = new analyzer_1.Analyzer();
        this.reportFeaturesFiles = this.reportFeaturesFiles.bind(this);
    }
    createGherkinsReport(config) {
        this.theme = config.theme;
        this.folderToReadReport = config.analysisFolder || this.folderToReadReport;
        this.folderToWriteReport = config.outputFolder || this.folderToWriteReport;
        if (this.folderToWriteReport.substr(-1) !== '/') {
            this.folderToWriteReport += '/';
        }
        this.setupReportFolder();
        this.reader.readFeatureFilesFromFolder(this.folderToReadReport, this.reportFeaturesFiles);
    }
    setupReportFolder() {
        if (fs.existsSync(this.folderToWriteReport)) {
            del.sync(this.folderToWriteReport);
        }
        fs.mkdirSync(this.folderToWriteReport, { recursive: true });
        fs.copyFileSync(`${this.folderToReadTemplates}${this.theme}.css`, `${this.folderToWriteReport}style.css`);
    }
    readAllGherkins(readFiles) {
        return __awaiter(this, void 0, void 0, function* () {
            const rowsFiles = [];
            yield Promise.all(readFiles
                .map((readFile) => __awaiter(this, void 0, void 0, function* () {
                const contentFile = yield this.reader.readContentFeatureFile(readFile);
                const rowsFile = this.reader.getRowsFeatureFile(contentFile);
                const gherkinsFile = this.analyzer.getGherkins(rowsFile);
                const dataFile = Object.assign({}, gherkinsFile, { file: readFile.replace(process.cwd(), '.') });
                rowsFiles.push(dataFile);
            })));
            return rowsFiles;
        });
    }
    readAllTemplates() {
        return __awaiter(this, void 0, void 0, function* () {
            const templatesNames = ['meta', 'menu', 'footer', 'files', 'features', 'scenarios', 'states', 'actions', 'outcomes'];
            const templates = {};
            yield Promise.all(templatesNames
                .map((templateName) => __awaiter(this, void 0, void 0, function* () {
                const readTemplate = yield this.reader.readContentFeatureFile(`${this.folderToReadTemplates}${templateName}.mustache`);
                templates[templateName] = readTemplate;
                return {
                    [templateName]: readTemplate
                };
            })));
            return templates;
        });
    }
    prepareReports() {
        this.templatesView = {
            list: this.gherkins,
            date: moment().format('YYYY/MM/DD'),
            time: moment().format('HH:mm:ss')
        };
        this.templatePartials = {
            meta: this.templates.meta,
            footer: this.templates.footer,
            menu: this.templates.menu
        };
    }
    writeFilesReport() {
        const reportFilesList = Mustache.render(this.templates.files, this.templatesView, this.templatePartials);
        fs.writeFile(`${this.folderToWriteReport}index.html`, reportFilesList, writeError => {
            if (writeError) {
                console.error(writeError);
                process.exit(1);
                return;
            }
        });
    }
    writeFeaturesReport() {
        const reportFeaturesList = Mustache.render(this.templates.features, this.templatesView, this.templatePartials);
        fs.writeFile(`${this.folderToWriteReport}features.html`, reportFeaturesList, writeError => {
            if (writeError) {
                console.error(writeError);
                process.exit(1);
                return;
            }
        });
    }
    writeScenariosReport() {
        const reportScenariosList = Mustache.render(this.templates.scenarios, this.templatesView, this.templatePartials);
        fs.writeFile(`${this.folderToWriteReport}scenarios.html`, reportScenariosList, writeError => {
            if (writeError) {
                console.error(writeError);
                process.exit(1);
                return;
            }
        });
    }
    writeStatesReport() {
        const reportStatesList = Mustache.render(this.templates.states, this.templatesView, this.templatePartials);
        fs.writeFile(`${this.folderToWriteReport}states.html`, reportStatesList, writeError => {
            if (writeError) {
                console.error(writeError);
                process.exit(1);
                return;
            }
        });
    }
    writeActionsReport() {
        const reportActionsList = Mustache.render(this.templates.actions, this.templatesView, this.templatePartials);
        fs.writeFile(`${this.folderToWriteReport}actions.html`, reportActionsList, writeError => {
            if (writeError) {
                console.error(writeError);
                process.exit(1);
                return;
            }
        });
    }
    writeOutcomesReport() {
        const reportOutcomesList = Mustache.render(this.templates.outcomes, this.templatesView, this.templatePartials);
        fs.writeFile(`${this.folderToWriteReport}outcomes.html`, reportOutcomesList, writeError => {
            if (writeError) {
                console.error(writeError);
                process.exit(1);
                return;
            }
        });
    }
    reportFeaturesFiles(readError, readFiles) {
        return __awaiter(this, void 0, void 0, function* () {
            if (readError) {
                console.error(readError);
                process.exit(1);
                return;
            }
            this.gherkins = yield this.readAllGherkins(readFiles);
            this.templates = yield this.readAllTemplates();
            this.prepareReports();
            this.writeFilesReport();
            this.writeFeaturesReport();
            this.writeScenariosReport();
            this.writeStatesReport();
            this.writeActionsReport();
            this.writeOutcomesReport();
        });
    }
}
exports.Reporter = Reporter;
